@@ -1,7 +1,10 @@
+# coding: utf-8
 """test_isort.py.
 
 Tests all major functionality of the isort library
-Should be ran using py.test by simply running by.test in the isort project directory
+Should be ran using py.test by simply running py.test in the isort project directory
+
+Copyright (C) 2013  Timothy Edmund Crosley
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation
@@ -20,11 +23,16 @@ OTHER DEALINGS IN THE SOFTWARE.
 """
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from pies.overrides import *
+import codecs
+import os
+import shutil
+import tempfile
 
 from isort.isort import SortImports
+from isort.pie_slice import *
 from isort.settings import WrapModes
 
+SHORT_IMPORT = "from third_party import lib1, lib2, lib3, lib4"
 REALLY_LONG_IMPORT = ("from third_party import lib1, lib2, lib3, lib4, lib5, lib6, lib7, lib8, lib9, lib10, lib11,"
                       "lib12, lib13, lib14, lib15, lib16, lib17, lib18, lib20, lib21, lib22")
 REALLY_LONG_IMPORT_WITH_COMMENT = ("from third_party import lib1, lib2, lib3, lib4, lib5, lib6, lib7, lib8, lib9, "
@@ -139,6 +147,29 @@ def test_line_length():
                            "                         lib18, lib20,\n"
                            "                         lib21, lib22)\n")
 
+    test_output = SortImports(file_contents=REALLY_LONG_IMPORT, line_length=42, wrap_length=32).output
+    assert test_output == ("from third_party import (lib1,\n"
+                           "                         lib2,\n"
+                           "                         lib3,\n"
+                           "                         lib4,\n"
+                           "                         lib5,\n"
+                           "                         lib6,\n"
+                           "                         lib7,\n"
+                           "                         lib8,\n"
+                           "                         lib9,\n"
+                           "                         lib10,\n"
+                           "                         lib11,\n"
+                           "                         lib12,\n"
+                           "                         lib13,\n"
+                           "                         lib14,\n"
+                           "                         lib15,\n"
+                           "                         lib16,\n"
+                           "                         lib17,\n"
+                           "                         lib18,\n"
+                           "                         lib20,\n"
+                           "                         lib21,\n"
+                           "                         lib22)\n")
+
 
 def test_output_modes():
     """Test setting isort to use various output modes works as expected"""
@@ -314,6 +345,19 @@ def test_output_modes():
                                                     "    lib17, lib18, lib20, lib21, lib22\n"
                                                     ")\n")
 
+    output_noqa = SortImports(file_contents=REALLY_LONG_IMPORT_WITH_COMMENT,
+                              multi_line_output=WrapModes.NOQA).output
+    assert output_noqa == "from third_party import lib1 lib2 lib3 lib4 lib5 lib6 lib7 lib8 lib9 lib10 lib11 lib12 lib13 lib14 lib15 lib16 lib17 lib18 lib20 lib21 lib22  # NOQA comment\n"  # NOQA
+
+
+def test_qa_comment_case():
+    test_input = "from veryveryveryveryveryveryveryveryveryveryvery import X  # NOQA"
+    test_output = SortImports(file_contents=test_input, line_length=40, multi_line_output=WrapModes.NOQA).output
+    assert test_output == "from veryveryveryveryveryveryveryveryveryveryvery import X  # NOQA\n"
+
+    test_input = "import veryveryveryveryveryveryveryveryveryveryvery  # NOQA"
+    test_output = SortImports(file_contents=test_input, line_length=40, multi_line_output=WrapModes.NOQA).output
+    assert test_output == "import veryveryveryveryveryveryveryveryveryveryvery  # NOQA\n"
 
 def test_length_sort():
     """Test setting isort to sort on length instead of alphabetically."""
@@ -386,6 +430,18 @@ def test_custom_indent():
                            "  lib20, lib21, lib22\n")
 
 
+def test_use_parentheses():
+    test_input = (
+        "from fooooooooooooooooooooooooo.baaaaaaaaaaaaaaaaaaarrrrrrr import \\"
+        "    my_custom_function as my_special_function"
+    )
+    test_output = SortImports(
+        file_contents=test_input, known_third_party=['django'],
+        line_length=79, use_parentheses=True,
+    ).output
+    assert '(' in test_output
+
+
 def test_skip():
     """Ensure skipping a single import will work as expected."""
     test_input = ("import myproject\n"
@@ -402,6 +458,16 @@ def test_skip():
                            "import sys  # isort:skip this import needs to be placed here\n")
 
 
+def test_skip_with_file_name():
+    """Ensure skipping a file works even when file_contents is provided."""
+    test_input = ("import django\n"
+                  "import myproject\n")
+
+    skipped = SortImports(file_path='/baz.py', file_contents=test_input, known_third_party=['django'],
+                          skip=['baz.py']).skipped
+    assert skipped
+
+
 def test_force_to_top():
     """Ensure forcing a single import to the top of its category works as expected."""
     test_input = ("import lib6\n"
@@ -499,7 +565,6 @@ def test_remove_imports():
                            "import lib5\n")
 
 
-
 def test_explicitly_local_import():
     """Ensure that explicitly local imports are separated."""
     test_input = ("import lib1\n"
@@ -615,6 +680,7 @@ def test_forced_separate():
     assert SortImports(file_contents=test_input, forced_separate=['.y'],
                        line_length=120, order_by_type=False).output == test_input
 
+
 def test_default_section():
     """Test to ensure changing the default section works as expected."""
     test_input = ("import sys\n"
@@ -639,6 +705,30 @@ def test_default_section():
                                   "import django.settings\n")
 
 
+def test_first_party_overrides_standard_section():
+    """Test to ensure changing the default section works as expected."""
+    test_input = ("import sys\n"
+                  "import os\n"
+                  "import profile.test\n")
+    test_output = SortImports(file_contents=test_input, known_first_party=['profile']).output
+    assert test_output == ("import os\n"
+                           "import sys\n"
+                           "\n"
+                           "import profile.test\n")
+
+
+def test_thirdy_party_overrides_standard_section():
+    """Test to ensure changing the default section works as expected."""
+    test_input = ("import sys\n"
+                  "import os\n"
+                  "import profile.test\n")
+    test_output = SortImports(file_contents=test_input, known_third_party=['profile']).output
+    assert test_output == ("import os\n"
+                           "import sys\n"
+                           "\n"
+                           "import profile.test\n")
+
+
 def test_force_single_line_imports():
     """Test to ensure forcing imports to each have their own line works as expected."""
     test_input = ("from third_party import lib1, lib2, \\\n"
@@ -671,6 +761,13 @@ def test_force_single_line_imports():
                            "from third_party import lib22\n")
 
 
+def test_force_single_line_long_imports():
+    test_input = ("from veryveryveryveryveryvery import small, big\n")
+    test_output = SortImports(file_contents=test_input, multi_line_output=WrapModes.NOQA,
+                              line_length=40, force_single_line=True).output
+    assert test_output == ("from veryveryveryveryveryvery import big\n"
+                           "from veryveryveryveryveryvery import small  # NOQA\n")
+
 def test_titled_imports():
     """Tests setting custom titled/commented import sections."""
     test_input = ("import sys\n"
@@ -918,6 +1015,7 @@ def test_keep_comments():
         "from a import b as bb, c as cc, d  # b comment; c comment\n"
     )
 
+
 def test_multiline_split_on_dot():
     """Test to ensure isort correctly handles multiline imports, even when split right after a '.'"""
     test_input = ("from my_lib.my_package.test.level_1.level_2.level_3.level_4.level_5.\\\n"
@@ -936,6 +1034,71 @@ def test_import_star():
     assert SortImports(file_contents=test_input, combine_star=True).output == ("from blah import *\n")
 
 
+def test_include_trailing_comma():
+    """Test for the include_trailing_comma option"""
+    test_output_grid = SortImports(
+        file_contents=SHORT_IMPORT,
+        multi_line_output=WrapModes.GRID,
+        line_length=40,
+        include_trailing_comma=True,
+    ).output
+    assert test_output_grid == (
+        "from third_party import (lib1, lib2,\n"
+        "                         lib3, lib4,)\n"
+    )
+
+    test_output_vertical = SortImports(
+        file_contents=SHORT_IMPORT,
+        multi_line_output=WrapModes.VERTICAL,
+        line_length=40,
+        include_trailing_comma=True,
+    ).output
+    assert test_output_vertical == (
+        "from third_party import (lib1,\n"
+        "                         lib2,\n"
+        "                         lib3,\n"
+        "                         lib4,)\n"
+    )
+
+    test_output_vertical_indent = SortImports(
+        file_contents=SHORT_IMPORT,
+        multi_line_output=WrapModes.VERTICAL_HANGING_INDENT,
+        line_length=40,
+        include_trailing_comma=True,
+    ).output
+    assert test_output_vertical_indent == (
+        "from third_party import (\n"
+        "    lib1,\n"
+        "    lib2,\n"
+        "    lib3,\n"
+        "    lib4,\n"
+        ")\n"
+    )
+
+    test_output_vertical_grid = SortImports(
+        file_contents=SHORT_IMPORT,
+        multi_line_output=WrapModes.VERTICAL_GRID,
+        line_length=40,
+        include_trailing_comma=True,
+    ).output
+    assert test_output_vertical_grid == (
+        "from third_party import (\n"
+        "    lib1, lib2, lib3, lib4,)\n"
+    )
+
+    test_output_vertical_grid_grouped = SortImports(
+        file_contents=SHORT_IMPORT,
+        multi_line_output=WrapModes.VERTICAL_GRID_GROUPED,
+        line_length=40,
+        include_trailing_comma=True,
+    ).output
+    assert test_output_vertical_grid_grouped == (
+        "from third_party import (\n"
+        "    lib1, lib2, lib3, lib4,\n"
+        ")\n"
+    )
+
+
 def test_similar_to_std_library():
     """Test to ensure modules that are named similarly to a standard library import don't end up clobbered"""
     test_input = ("import datetime\n"
@@ -1081,3 +1244,397 @@ def test_place_comments():
                            "# isort:imports-stdlib\n"
                            "import os\n"
                            "import sys\n")
+
+
+def test_placement_control():
+    """Ensure that most specific placement control match wins"""
+    test_input = ("import os\n"
+                  "import sys\n"
+                  "from bottle import Bottle, redirect, response, run\n"
+                  "import p24.imports._argparse as argparse\n"
+                  "import p24.imports._subprocess as subprocess\n"
+                  "import p24.imports._VERSION as VERSION\n"
+                  "import p24.shared.media_wiki_syntax as syntax\n")
+    test_output = SortImports(file_contents=test_input,
+                known_first_party=['p24', 'p24.imports._VERSION'],
+                known_standard_library=['p24.imports'],
+                known_third_party=['bottle'],
+                default_section="THIRDPARTY").output
+
+    assert test_output == ("import os\n"
+                           "import p24.imports._argparse as argparse\n"
+                           "import p24.imports._subprocess as subprocess\n"
+                           "import sys\n"
+                           "\n"
+                           "from bottle import Bottle, redirect, response, run\n"
+                           "\n"
+                           "import p24.imports._VERSION as VERSION\n"
+                           "import p24.shared.media_wiki_syntax as syntax\n")
+
+
+def test_custom_sections():
+    """Ensure that most specific placement control match wins"""
+    test_input = ("import os\n"
+                  "import sys\n"
+                  "from django.conf import settings\n"
+                  "from bottle import Bottle, redirect, response, run\n"
+                  "import p24.imports._argparse as argparse\n"
+                  "from django.db import models\n"
+                  "import p24.imports._subprocess as subprocess\n"
+                  "import pandas as pd\n"
+                  "import p24.imports._VERSION as VERSION\n"
+                  "import numpy as np\n"
+                  "import p24.shared.media_wiki_syntax as syntax\n")
+    test_output = SortImports(file_contents=test_input,
+                known_first_party=['p24', 'p24.imports._VERSION'],
+                import_heading_stdlib='Standard Library',
+                import_heading_thirdparty='Third Party',
+                import_heading_firstparty='First Party',
+                import_heading_django='Django',
+                import_heading_pandas='Pandas',
+                known_standard_library=['p24.imports'],
+                known_third_party=['bottle'],
+                known_django=['django'],
+                known_pandas=['pandas', 'numpy'],
+                default_section="THIRDPARTY",
+                sections=["FUTURE", "STDLIB", "DJANGO", "THIRDPARTY", "PANDAS", "FIRSTPARTY", "LOCALFOLDER"]).output
+    assert test_output == ("# Standard Library\n"
+                           "import os\n"
+                           "import p24.imports._argparse as argparse\n"
+                           "import p24.imports._subprocess as subprocess\n"
+                           "import sys\n"
+                           "\n"
+                           "# Django\n"
+                           "from django.conf import settings\n"
+                           "from django.db import models\n"
+                           "\n"
+                           "# Third Party\n"
+                           "from bottle import Bottle, redirect, response, run\n"
+                           "\n"
+                           "# Pandas\n"
+                           "import numpy as np\n"
+                           "import pandas as pd\n"
+                           "\n"
+                           "# First Party\n"
+                           "import p24.imports._VERSION as VERSION\n"
+                           "import p24.shared.media_wiki_syntax as syntax\n")
+
+
+def test_sticky_comments():
+    """Test to ensure it is possible to make comments 'stick' above imports"""
+    test_input = ("import os\n"
+                  "\n"
+                  "# Used for type-hinting (ref: https://github.com/davidhalter/jedi/issues/414).\n"
+                  "from selenium.webdriver.remote.webdriver import WebDriver  # noqa\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+    test_input = ("from django import forms\n"
+                  "# While this couples the geographic forms to the GEOS library,\n"
+                  "# it decouples from database (by not importing SpatialBackend).\n"
+                  "from django.contrib.gis.geos import GEOSException, GEOSGeometry\n"
+                  "from django.utils.translation import ugettext_lazy as _\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_zipimport():
+    """Imports ending in "import" shouldn't be clobbered"""
+    test_input = "from zipimport import zipimport\n"
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_from_ending():
+    """Imports ending in "from" shouldn't be clobbered."""
+    test_input = "from foo import get_foo_from, get_foo\n"
+    expected_output = "from foo import get_foo, get_foo_from\n"
+    assert SortImports(file_contents=test_input).output == expected_output
+
+
+def test_from_first():
+    """Tests the setting from_first works correctly"""
+    test_input = "from os import path\nimport os\n"
+    assert SortImports(file_contents=test_input, from_first=True).output == test_input
+
+
+def test_top_comments():
+    """Ensure correct behavior with top comments"""
+    test_input = ("# -*- encoding: utf-8 -*-\n"
+                  "# Test comment\n"
+                  "#\n"
+                  "from __future__ import unicode_literals\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+    test_input = ("# -*- coding: utf-8 -*-\n"
+                  "from django.db import models\n"
+                  "from django.utils.encoding import python_2_unicode_compatible\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+    test_input = ("# Comment\n"
+                  "import sys\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+    test_input = ("# -*- coding\n"
+                  "import sys\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_consistency():
+    """Ensures consistency of handling even when dealing with non ordered-by-type imports"""
+    test_input = "from sqlalchemy.dialects.postgresql import ARRAY, array\n"
+    assert SortImports(file_contents=test_input, order_by_type=True).output == test_input
+
+
+def test_force_grid_wrap():
+    """Ensures removing imports works as expected."""
+    test_input = (
+      "from foo import lib6, lib7\n"
+      "from bar import lib2\n"
+    )
+    test_output = SortImports(
+      file_contents=test_input,
+      force_grid_wrap=True,
+      multi_line_output=WrapModes.VERTICAL_HANGING_INDENT
+      ).output
+    assert test_output == """from bar import lib2
+from foo import (
+    lib6,
+    lib7
+)
+"""
+
+
+def test_force_grid_wrap_long():
+    """Ensure that force grid wrap still happens with long line length"""
+    test_input = (
+      "from foo import lib6, lib7\n"
+      "from bar import lib2\n"
+      "from babar import something_that_is_kind_of_long"
+    )
+    test_output = SortImports(
+      file_contents=test_input,
+      force_grid_wrap=True,
+      multi_line_output=WrapModes.VERTICAL_HANGING_INDENT,
+      line_length=9999,
+      ).output
+    assert test_output == """from babar import something_that_is_kind_of_long
+from bar import lib2
+from foo import (
+    lib6,
+    lib7
+)
+"""
+
+
+def test_uses_jinja_variables():
+    """Test a basic set of imports that use jinja variables"""
+    test_input = ("import sys\n"
+                  "import os\n"
+                  "import myproject.{ test }\n"
+                  "import django.{ settings }")
+    test_output = SortImports(file_contents=test_input, known_third_party=['django'],
+                              known_first_party=['myproject']).output
+    assert test_output == ("import os\n"
+                           "import sys\n"
+                           "\n"
+                           "import django.{ settings }\n"
+                           "\n"
+                           "import myproject.{ test }\n")
+
+    test_input = ("import {{ cookiecutter.repo_name }}\n"
+                  "from foo import {{ cookiecutter.bar }}\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_fcntl():
+    """Test to ensure fcntl gets correctly recognized as stdlib import"""
+    test_input = ("import fcntl\n"
+                  "import os\n"
+                  "import sys\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_import_split_is_word_boundary_aware():
+    """Test to ensure that isort splits words in a boundry aware mannor"""
+    test_input = ("from mycompany.model.size_value_array_import_func import ("
+                "    get_size_value_array_import_func_jobs,"
+                ")")
+    test_output = SortImports(file_contents=test_input,
+      multi_line_output=WrapModes.VERTICAL_HANGING_INDENT,
+      line_length=79).output
+
+    assert test_output == ("from mycompany.model.size_value_array_import_func import \\\n"
+                           "    get_size_value_array_import_func_jobs\n")
+
+
+def test_other_file_encodings():
+    """Test to ensure file encoding is respected"""
+    try:
+        tmp_dir = tempfile.mkdtemp()
+        for encoding in ('latin1', 'utf8'):
+            tmp_fname = os.path.join(tmp_dir, 'test_{0}.py'.format(encoding))
+            with codecs.open(tmp_fname, mode='w', encoding=encoding) as f:
+                file_contents = "# coding: {0}\n\ns = u'ã'\n".format(encoding)
+                f.write(file_contents)
+        assert SortImports(file_path=tmp_fname).output == file_contents
+    finally:
+        shutil.rmtree(tmp_dir, ignore_errors=True)
+
+
+def test_comment_at_top_of_file():
+    """Test to ensure isort correctly handles top of file comments"""
+    test_input = ("# Comment one\n"
+                  "from django import forms\n"
+                  "# Comment two\n"
+                  "from django.contrib.gis.geos import GEOSException\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+    test_input = ("# -*- coding: utf-8 -*-\n"
+                  "from django.db import models\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_alphabetic_sorting():
+    """Test to ensure isort correctly handles top of file comments"""
+    test_input = ("from django.contrib.gis.geos import GEOSException\n"
+                  "from plone.app.testing import getRoles\n"
+                  "from plone.app.testing import ManageRoles\n"
+                  "from plone.app.testing import setRoles\n"
+                  "from Products.CMFPlone import utils\n"
+                  "\n"
+                  "import ABC\n"
+                  "import unittest\n"
+                  "import Zope\n")
+    options = {'force_single_line': True,
+               'force_alphabetical_sort': True, }
+    assert SortImports(file_contents=test_input, **options).output == test_input
+
+    test_input = ("# -*- coding: utf-8 -*-\n"
+                  "from django.db import models\n")
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_comments_not_duplicated():
+    """Test to ensure comments aren't duplicated: issue 303"""
+    test_input = ('from flask import url_for\n'
+                  "# Whole line comment\n"
+                  'from service import demo  # inline comment\n'
+                  'from service import settings\n')
+    output = SortImports(file_contents=test_input).output
+    assert output.count("# Whole line comment\n") == 1
+    assert output.count("# inline comment\n") == 1
+
+
+def test_top_of_line_comments():
+    """Test to ensure top of line comments stay where they should: issue 260"""
+    test_input = ('# -*- coding: utf-8 -*-\n'
+                  'from django.db import models\n'
+                  '#import json as simplejson\n'
+                  'from myproject.models import Servidor\n'
+                  '\n'
+                  'import reversion\n'
+                   '\n'
+                   'import logging\n')
+    output = SortImports(file_contents=test_input).output
+    assert output.startswith('# -*- coding: utf-8 -*-\n')
+
+
+def test_basic_comment():
+    """Test to ensure a basic comment wont crash isort"""
+    test_input = ('import logging\n'
+                  '# Foo\n'
+                  'import os\n')
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_shouldnt_add_lines():
+    """Ensure that isort doesn't add a blank line when a top of import comment is present, issue #316"""
+    test_input = ('"""Text"""\n'
+                  '# This is a comment\n'
+                 'import pkg_resources\n')
+    assert SortImports(file_contents=test_input).output == test_input
+
+
+def test_sections_parsed_correct():
+    """Ensure that modules for custom sections parsed as list from config file and isort result is correct"""
+    tmp_conf_dir = None
+    conf_file_data = (
+        '[settings]\n'
+        'sections=FUTURE,STDLIB,THIRDPARTY,FIRSTPARTY,LOCALFOLDER,COMMON\n'
+        'known_common=nose\n'
+        'import_heading_common=Common Library\n'
+        'import_heading_stdlib=Standard Library\n'
+    )
+    test_input = (
+        'import os\n'
+        'from nose import *\n'
+        'import nose\n'
+        'from os import path'
+    )
+    correct_output = (
+        '# Standard Library\n'
+        'import os\n'
+        'from os import path\n'
+        '\n'
+        '# Common Library\n'
+        'import nose\n'
+        'from nose import *\n'
+    )
+
+    try:
+        tmp_conf_dir = tempfile.mkdtemp()
+        tmp_conf_name = os.path.join(tmp_conf_dir, '.isort.cfg')
+        with codecs.open(tmp_conf_name, 'w') as test_config:
+            test_config.writelines(conf_file_data)
+
+        assert SortImports(file_contents=test_input, settings_path=tmp_conf_dir).output == correct_output
+    finally:
+        shutil.rmtree(tmp_conf_dir, ignore_errors=True)
+
+
+def test_alphabetic_sorting_no_newlines():
+    '''Test to ensure that alphabetical sort does not erroneously introduce new lines (issue #328)'''
+    test_input = "import os\n"
+    test_output = SortImports(file_contents=test_input,force_alphabetical_sort=True).output
+    assert test_input == test_output
+
+    test_input = ('from a import b\n'
+                  '\n'
+                  'import os\n'
+                  'import unittest\n'
+                  '\n'
+                  '\n'
+                  'print(1)\n')
+    test_output = SortImports(file_contents=test_input,force_alphabetical_sort=True, lines_after_imports=2).output
+    assert test_input == test_output
+
+
+def test_sort_within_section():
+    '''Test to ensure its possible to force isort to sort within sections'''
+    test_input = ('from Foob import ar\n'
+                  'import foo\n'
+                  'from foo import bar\n'
+                  'from foo.bar import Quux, baz\n')
+    test_output = SortImports(file_contents=test_input,force_sort_within_sections=True).output
+    assert test_output == test_input
+
+    test_input = ('import foo\n'
+                  'from foo import bar\n'
+                  'from foo.bar import baz\n'
+                  'from foo.bar import Quux\n'
+                  'from Foob import ar\n')
+    test_output = SortImports(file_contents=test_input,force_sort_within_sections=True, order_by_type=False,
+                              force_single_line=True).output
+    assert test_output == test_input
+
+
+def test_sorting_with_two_top_comments():
+    '''Test to ensure isort will sort files that contain 2 top comments'''
+    test_input = ('#! comment1\n'
+                  "''' comment2\n"
+                  "'''\n"
+                  'import b\n'
+                  'import a\n')
+    assert SortImports(file_contents=test_input).output == ('#! comment1\n'
+                                                            "''' comment2\n"
+                                                            "'''\n"
+                                                            'import a\n'
+                                                            'import b\n')
